using System;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Grouping;

namespace GGCdynamiccontextMenu
{
	/// <summary>
	/// GroupingEnginePFFactoryPF provides a trimmed down GridGroup
	/// which eliminates overhead of not needed preview rows, header and footer cells.
	/// </summary>
	public class GroupingEngineFactoryPF : GridEngineFactoryBase
	{
		// Add this line in your forms ctor:
		// GroupingEnginePFFactoryPF provides custom group that does not display caption if only one row in group
		// GridEngineFactory.Factory = new GroupingEnginePFFactoryPF();

		public override GridEngine CreateEngine()
		{
			return new GroupingEnginePF();
		}
	}


	public class GroupingTableDescriptorPF : GridTableDescriptor
	{
		public GroupingTableDescriptorPF(GridEngine engine, RelationDescriptor parentRelation)
			: base(engine, parentRelation)
		{
		}

		public override void EnsureSummaryDescriptors()
		{
			if (this.ParentRelation != null && this.ParentRelation.RelationKind == RelationKind.ForeignKeyKeyWords)
				return;

			base.EnsureSummaryDescriptors();
		}

	}

	public class GroupingRelationDescriptorPF : GridRelationDescriptor
	{
		/// <summary>
		/// Initializes a new empty relation descriptor.
		/// </summary>
		public GroupingRelationDescriptorPF()
		{
		}

		/// <summary>
		/// Initializes a new relation descriptor with the given name.
		/// </summary>
		/// <param name="relationName"></param>
		public GroupingRelationDescriptorPF(string relationName)
			: base(relationName)
		{
		}

		/// <override/>
		public override TableDescriptor CreateChildTableDescriptor()
		{
			return this.ParentTableDescriptor.Engine.CreateTableDescriptor(this);
		}
	}

	public class GroupingEnginePF : GridEngine
	{
		public GroupingEnginePF()
		{
		}

		public override RelationDescriptor CreateRelationDescriptor()
		{
			return new GroupingRelationDescriptorPF();
		}

		public override TableDescriptor CreateTableDescriptor(RelationDescriptor parentRelation)
		{
			return new GroupingTableDescriptorPF(this, parentRelation);
		}

		public override Group CreateGroup(Section parent)
		{
			return new GroupingGroupPF(parent);
		}

		public override RecordRow CreateRecordRow(RecordRowsPart parent)
		{
			return new GroupingRecordRowPF(parent);
		}

		public override CaptionRow CreateCaptionRow(CaptionSection parent)
		{
			return new GroupingCaptionRowPF (parent);
		}

		public override ColumnHeaderRow CreateColumnHeaderRow(ColumnHeaderSection parent)
		{
			return new GroupingColumnHeaderRowPF (parent);
		}

	}

	public class GroupingGroupPF : Group, IGridGroupOptionsSource
	{
		public GroupingGroupPF(Section parent)
			: base(parent)
		{
		}

		/// <summary>
		/// Add in the sections
		/// </summary>
		/// <param name="hasRecords"></param>
		/// <param name="fields"></param>
		protected override void OnInitializeSections(bool hasRecords, SortColumnDescriptorCollection fields)
		{
			// Caption
			this.Sections.Add(this.ParentTableDescriptor.CreateCaptionSection(this));

			// Details (Records or Groups)
			if (hasRecords)
				this.Sections.Add(this.ParentTableDescriptor.CreateRecordsDetails(this, fields));
			else
				this.Sections.Add(this.ParentTableDescriptor.CreateGroupsDetails(this, fields));

			this.Sections.Add(this.ParentTableDescriptor.CreateSummarySection(this));
		}		


		/// <summary>
		/// Hide groupings with only one entry
		/// </summary>
		/// <param name="el"></param>
		/// <returns>true if the element is to be displayed</returns>
		public override bool IsChildVisible(Element el)
		{
			if (el is CaptionSection)
			{
				if (this.Details == null || this.Records.Count == 0 || this.Records.Count > 1)
					return true;
				else
					return false;
			}
			else if (el is DetailsSection)
			{
				if (this.Details == null || this.Records.Count == 0 || this.Records.Count > 1)
					return el.ParentGroup.IsExpanded;
				else
					return true;
			}	
			else if (el is GridSummarySection)
			{
				if (this.Details == null || this.Records.Count == 0 || this.Records.Count > 1)
					return el.ParentGroup.IsExpanded;
				else
					return false;
			}		
			else 
				return el.ParentGroup.IsExpanded;
		}


		#region IGridGroupOptionsSource Members

		GridGroupOptionsStyleInfo groupOptions;

		public void RaiseGroupOptionsChanging(GridGroupOptionsChangedEventArgs e)
		{
		}

		public void RaiseGroupOptionsChanged(GridGroupOptionsChangedEventArgs e)
		{
			this.InvalidateCounterTopDown(true);
		}

		public bool HasGroupOptions
		{
			get
			{
				return groupOptions != null;
			}
		}

		/// <override/>
		public new GridEngine Engine
		{
			get
			{
				return (GridEngine) base.Engine;
			}
		}

		/// <summary>
		/// Lets you control the look of inner groups like whether the Caption Row is visible, or what CaptionText is.
		/// </summary>
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Content)]
		[Browsable(true)]
		public GridGroupOptionsStyleInfo GroupOptions
		{
			get
			{
				if (groupOptions == null)
				{
					GridGroupOptionsType t = GridGroupOptionsType.Groups;
					if (this.IsTopLevelGroup)
						t = GridGroupOptionsType.TopLevelGroup;
					else //if (this is ChildTable)
						t = GridGroupOptionsType.ChildTable;

					groupOptions = new GridGroupOptionsStyleInfo(new GridGroupOptionsStyleInfoIdentity((IGridGroupOptionsSource) this, t));
				}
				return groupOptions;
			}
			set
			{
				GroupOptions.CopyFrom(value);
			}
		}

		public IGridGroupOptionsSource GetParentGroupOptionsSource()
		{
			if (this.CategoryColumns.Count > 0)
			{
				GridColumnDescriptor column = ((GridTableDescriptor) this.ParentTableDescriptor).Columns[this.CategoryColumns[0].Name];
				if (column != null)
					return column;
			}
			return (IGridGroupOptionsSource) this.ParentTableDescriptor;
		}

		#endregion
	}


	public class GroupingRecordRowPF : GridRecordRow, IGridRowHeight
	{
		int rowHeight = -1;

		/// <summary>
		/// Initializes a new object in the specifed record part.
		/// </summary>
		/// <param name="parent">The parent element.</param>
		public GroupingRecordRowPF(RecordRowsPart parent)
			: base(parent)
		{
		}
		#region IGridRowHeight Members

		/// <summary>
		/// Determines if elements supports storing row heights 
		/// </summary>
		/// <returns></returns>
		public bool SupportsRowHeight()
		{
			return true;
		}

		/// <summary>
		/// The row height 
		/// </summary>
		public int RowHeight
		{
			get
			{
				return rowHeight;
			}
			set
			{
				if (rowHeight != value)
					rowHeight = value;
			}
		}

		/// <summary>
		/// Checks if row height was modified or if default setting should be used.
		/// </summary>
		public bool HasRowHeight
		{
			get
			{
				return rowHeight != -1;
			}
		}

		#endregion

		/// <summary>
		/// This is where the row height then gets integrated with the engine
		/// YAmount Counter logic.
		/// </summary>
		/// <returns></returns>
		public override double GetYAmountCount()
		{
			// Note: whenever the value that is returned by GetYAmountCount changes
			// make sure you call InvalidateCounterBottomUp so that the engine
			// is aware of the change and counters are recalculated. See
			// the RowHeight setter. 
			return rowHeight != -1 ? rowHeight : base.GetYAmountCount();
		}

	}


	public class GroupingCaptionRowPF : GridCaptionRow, IGridRowHeight
	{
		int rowHeight = -1;

		/// <summary>
		/// Initializes a new object in the specifed record part.
		/// </summary>
		/// <param name="parent">The parent element.</param>
		public GroupingCaptionRowPF(CaptionSection parent)
			: base(parent)
		{
		}
		#region IGridRowHeight Members

		/// <summary>
		/// Determines if elements supports storing row heights 
		/// </summary>
		/// <returns></returns>
		public bool SupportsRowHeight()
		{
			return true;
		}

		/// <summary>
		/// The row height 
		/// </summary>
		public int RowHeight
		{
			get
			{
				return rowHeight;
			}
			set
			{
				if (rowHeight != value)
				{
					rowHeight = value;
					// when Y-Amount scrolling is disabled, this is obsolete:
					// this.InvalidateCounterBottomUp();
				}
			}
		}

		/// <summary>
		/// Checks if row height was modified or if default setting should be used.
		/// </summary>
		public bool HasRowHeight
		{
			get
			{
				return rowHeight != -1;
			}
		}

		#endregion

	
		/// <summary>
		/// This is where the row height then gets integrated with the engine
		/// YAmount Counter logic.
		/// </summary>
		/// <returns></returns>
		public override double GetYAmountCount()
		{
			// Note: whenever the value that is returned by GetYAmountCount changes
			// make sure you call InvalidateCounterBottomUp so that the engine
			// is aware of the change and counters are recalculated. See
			// the RowHeight setter. 
			return rowHeight != -1 ? rowHeight : base.GetYAmountCount();
		}

	}


	public class GroupingColumnHeaderRowPF : GridColumnHeaderRow, IGridRowHeight
	{
		int rowHeight = -1;

		/// <summary>
		/// Initializes a new object in the specifed record part.
		/// </summary>
		/// <param name="parent">The parent element.</param>
		public GroupingColumnHeaderRowPF(ColumnHeaderSection parent)
			: base(parent)
		{
		}
		#region IGridRowHeight Members

		/// <summary>
		/// Determines if elements supports storing row heights 
		/// </summary>
		/// <returns></returns>
		public bool SupportsRowHeight()
		{
			return true;
		}

		/// <summary>
		/// The row height 
		/// </summary>
		public int RowHeight
		{
			get
			{
				return rowHeight;
			}
			set
			{
				if (rowHeight != value)
				{
					rowHeight = value;
					// when Y-Amount scrolling is disabled, this is obsolete:
					// this.InvalidateCounterBottomUp();
				}
			}
		}

		/// <summary>
		/// Checks if row height was modified or if default setting should be used.
		/// </summary>
		public bool HasRowHeight
		{
			get
			{
				return rowHeight != -1;
			}
		}

		#endregion

	
		/// <summary>
		/// This is where the row height then gets integrated with the engine
		/// YAmount Counter logic.
		/// </summary>
		/// <returns></returns>
		public override double GetYAmountCount()
		{
			// Note: whenever the value that is returned by GetYAmountCount changes
			// make sure you call InvalidateCounterBottomUp so that the engine
			// is aware of the change and counters are recalculated. See
			// the RowHeight setter. 
			return rowHeight != -1 ? rowHeight : base.GetYAmountCount();
		}

	}

}
